import React from 'react';
import ReactDOM from 'react-dom';
import { scaleBand, scaleLinear, max, format } from 'd3';

import { useData } from './useData';

import { ContinentChart } from './ContinentChart';
import { AxisBottom } from './AxisBottom';

const width = 960;
const height = 500;
const margin = { top: 20, right: 170, bottom: 40, left: 200 };
const innerWidth = width - margin.right - margin.left;
const innerHeight = height - margin.top - margin.bottom;
const xLabelOffset = 25;
const xTitleOffset = 60;
const yLabelOffset = 15;

const App = () => {
  const data = useData();

  if (!data) {
    return <pre>Loading...</pre>;
  }

  const gValue = (d) => d.continent;
  const xValue = (d) => d.pop;
  const yValue = (d) => d.country;

  const gScale = scaleBand()
    .domain(data.map(gValue))
    .range([0, innerHeight])
    .paddingInner(0.1)
    .paddingOuter(0);

  const xScale = scaleLinear()
    .domain([0, max(data, xValue)])
    .range([0, innerWidth]);

  const xFormatSpec = format('.2s');
  const xLabelFormat = (s) => xFormatSpec(s).replace('G', 'B');

  return (
    <div className="App">
      <svg width={width} height={height}>
        <g transform={`translate(${margin.left}, ${margin.top})`}>
          <AxisBottom
            xScale={xScale}
            innerHeight={innerHeight}
            xLabelOffset={xLabelOffset}
            xLabelFormat={xLabelFormat}
            xTitleOffset={xTitleOffset}
          />

          {gScale.domain().map((cont) => {
            const yScale = scaleBand()
              .domain(data.filter((d) => d.continent === cont).map(yValue))
              .range([0, gScale.bandwidth()])
              .padding(0.1);

            const gHeight = gScale.bandwidth();

            return (
              <ContinentChart
                data={data.filter((d) => d.continent === cont)}
                xScale={xScale}
                xValue={xValue}
                xLabelFormat={xLabelFormat}
                yScale={yScale}
                yValue={yValue}
                yLabelOffset={yLabelOffset}
                innerWidth={innerWidth}
                innerHeight={gHeight}
                y={gScale(cont)}
                title={cont}
              />
            );
          })}
        </g>
      </svg>
    </div>
  );
};

ReactDOM.render(<App />, document.getElementById('root'));
